unit Gmlib;
{IO Routinen fr FITS Dateien}
// O. Nickel, Mainz 2005

interface
uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Menus, ComCtrls, ExtCtrls, StdCtrls;


const
  RWmode : Byte = 102;              (*filemode read/write*)
  //ReadAccess = 0;
  //WriteAccess = 1;
  //ReadWriteAccess = 2;
  ByteRead = 100;
  ByteWrite = 101;
  ByteReadWrite = 102;
  Progdir: string = '';
  WorkDir: string = '';


type
  LevelRec      = array[1..4] of record ll,ul: word end;
  ColorInfotype = array[1..4] of record
                                   ct,lt,ut: integer;
                                   inverted: boolean;
                                 end;
  PathStr       = string;
  charset       = set of char;

{math functions}
function max(I1, I2 : Integer) : Integer;
function maxword(w1,w2:word) : word;
  (* returns maximum of two integers *)
function MIN(I1, I2 : Integer) : Integer;
function minword(w1,w2:word): word;
  (* returns minimum of two integers *)

function getdual(Dim : word) : Byte;
  (* returns log. dualis of given pos. number <dim> *)

  (************)
  (* FILE I/O *)
  (************)

function filestatus(var filvar : file) : Integer;
(* returns status of file <filvar>:
   file not open : filestatus=-1
   file open     : filestatus= readAccess (0) or
                               writeAccess (1) or
                               readWriteAccess (2)
*)


function LOOKUP(var filvar : file; filename : String) : Integer;
(* assigns file variable <filvar> to DOS filename <filename>
   and tests for existence of file.
   returned function code:
    0 : file exists, can be opened
   -1 : critical device error
   -2 : file not found
   -5 : file found, but access denied
*)


function IOPEN(var filvar : file; mode : Integer) : Integer;
(* open file <filvar> for specified access:
   <mode> = ByteRead        : readonly byte-indexed
          = ByteWrite       : writeonly  "    "
          = ByteReadWrite   : read/write "    "
   returned function code:
    0 : file was opened
   -1 : critical device error
   -2 : file not found
   -5 : file found, but access denied
*)

function getFilesize(filename:string) : LongInt;
(* returns size of file in bytes*)


function IREADbyte(nbytes : Longint; var Buffer;
                   index : LongInt; var filvar : file) : LongInt;
(*read specified number <nbytes> of bytes from file <filvar>
       into buffer <BUFFER> located at file position
       specified by byte-index <index>
       File must be opened before calling this function
       by IOPEN
       *)

function IWRITbyte(nbytes : Longint; var Buffer;
                   index : LongInt; var filvar : file) : LongInt;
(* writes specified number <nbytes> of bytes from buffer <BUFFER>
   to file <filvar> located at file position
   specified by byte-index <index>
   File must be opened before calling this function
   by IOPEN
   returned function value: number of bytes written, if >0
*)

function ICREATE(var filvar : file; filename : String;
                 mode : Integer) : Integer;
(* creates file with filename <filename>,
   parameter <mode> must be one of the following:
             ByteReadWrite, ByteWrite     : byte mode *)

function FileCopy(infilename,outfilename:string) : Integer;
  {copies file with name <infilename> to specified file
   with name <outfilename>.
   Existing files are overwritten !
   Returned function code:
   0 : copy successful
   <0: error during copy }



{ STRING functions }
{------------------}

function stUpcase(s:shortstring):shortstring;
function loCase(ch:char):char;
function StLoCase(s:shortstring):shortstring;
procedure RemoveBlanks(var s : ShortString);
function Trim(S:Shortstring):Shortstring;
function justfilename(filename:string):string;
function justpathname(filename:string):string;

  { deletes blank characters in string}
function nocomma(s:string): string;
function extractword(n:byte; S:string; WordDelims:charset):string;

{various functions}

function AddBackslash(s:string):string;


  function getnumber(num:longint;digits:byte):string;
  { converts a number <num> in a string with <digits> digits}
  { <digits>=4: 0000,0001,..9999}

  function getFilenumber(num:word):string;
  { converts a number fom 0 to 999 in a three-digit string}
  { 000,001,..999}

implementation


  function MAX(I1, I2 : Integer) : Integer;
  begin
    if I1 >= I2 then max := I1
    else max := I2;
  end;

  function MIN(I1, I2 : Integer) : Integer;
  begin
    if I1 <= I2 then MIN := I1
    else MIN := I2;
  end;

  function minword(w1,w2:word): word;
  begin
    if w1<=w2 then minword:=w1
              else minword:=w2;
  end;

  function maxword(w1,w2:word) : word;
  begin
    if w1>=w2 then maxword :=w1
              else maxword :=w2;
  end;


  function getdual(Dim : word) : Byte;
    (* returns log. dualis of given pos. number <dim> *)
  var k : Byte;
  begin
    k := 0;
    repeat
      Dim := Dim shr 1; k := k+1;
    until Dim = 0;
    getdual := k-1;
  end;

  (****************)
  (* DATAFILE I/O *)
  (****************)

  function filestatus(var filvar : file) : Integer;
(* returns status of file <filvar>:
   file not open : filestatus=-1
   file open     : filestatus= readAccess (0) or
                               writeAccess (1) or
                               readWriteAccess (2)
*)
  var
    f : Tfilerec absolute filvar;

  begin
    if f.mode = fmclosed then filestatus := -1
    else if f.mode = fminput then filestatus := ByteRead
    else if f.mode = fmoutput then filestatus := ByteWrite
    else if f.mode = fminout then filestatus := ByteReadWrite
    else filestatus := -1;
  end;


  function LOOKUP(var filvar : file; filename : String) : Integer;
(* assigns file variable <filvar> to DOS filename <filename>
   and tests for existence of file.
   returned function code:
    0 : file exists, can be opened
   -1 : critical device error
   -2 : file not found
   -5 : file found, but access denied
*)
  var IOfehler : Integer;
  begin
    Assign(filvar, filename);
    filemode := 0;                (*read only*)
     (*$I-*) Reset(filvar); (*$I+*)
    IOfehler := IoResult;
    case IOfehler of
      1 : LOOKUP := -2;           (*file not found*)
      2..4 : LOOKUP := -1;
      5 : LOOKUP := -5;           (* access denied*)
      102 : LOOKUP := -1;
      150..199 : LOOKUP := -1;    (*critical errors*)
    else begin
        LOOKUP := 0;
        Close(filvar);
      end;
    end;
  end;


  function IOPEN(var filvar : file; mode : Integer) : Integer;
(* open file <filvar> for specified access:
   <mode> = ReadAccess      : readonly with 512-byte blocks
          = WriteAccess     : writeonly with 512-byte blocks
          = ReadWriteAccess : read/write 512-byte blocks
          = ByteRead        : readonly byte-indexed
          = ByteWrite       : writeonly  "    "
          = ByteReadWrite   : read/write "    "
   returned function code:
    0 : file was opened
   -1 : critical device error
   -2 : file not found
   -5 : file found, but access denied
*)
  var error : Integer;
  begin
    case mode of
      0..2 : begin                (*512-byte records*)
               filemode := mode;
           (*$I-*) Reset(filvar, 512);
             end;
      100..102 : begin            (*byte-indexed mode*)
                   filemode := mode-100;
           (*$I-*) Reset(filvar, 1);
                 end;
    else begin
        IOPEN := -1;
        Exit;
      end;
    end;
     (*$I+*)
    error := IoResult;
    case error of
      0 : IOPEN := 0;
      1 : IOPEN := -2;            (*file not found*)
      2..4 : IOPEN := -1;
      5 : IOPEN := -5;            (* access denied*)
      102 : IOPEN := -1;
      103 : IOPEN:=0; { ??? }
      150..199 : IOPEN := -1;     (*critical errors*)
    else IOPEN := -1;
    end;
    //? filemode := 2;
  end;

  function getFilesize(filename:string) : LongInt;
(* returns size of file in bytes*)
  var filvar:file;
  begin
    getFilesize := -1;
    assignfile(filvar,filename);
    if IOPEN(filvar,byteread) < 0 then Exit;
     (*$I-*) getFilesize := FileSize(filvar); Close(filvar); (*$I+*)
  end;

  function ICREATE(var filvar : file; filename : String;
                   mode : Integer) : Integer;
  begin
    Assign(filvar, filename);
    case mode of
      1..2 : begin                (*512-byte records*)
               filemode := mode;
           (*$I-*) Rewrite(filvar, 512);
             end;
      101..102 : begin            (*byte-indexed mode*)
                   filemode := mode-100;
           (*$I-*) Rewrite(filvar, 1);
                 end;
    else begin
        ICREATE := -1;
        Exit;
      end;
    end;
     (*$I+*) ICREATE := -IoResult;

  end;

  function IREADbyte(nbytes : Longint; var Buffer;
                     index : LongInt; var filvar : file) : LongInt;
(*read specified number <nbytes> of bytes from file <filvar>
       into buffer <BUFFER> located at file position
       specified by byte-index <index>
       File must be opened before calling this function
       by IOPEN
       *)
  var
    error : Integer;
    nresult : Longint;

  begin
    nresult := 0;
    filemode:=0;
     (*$I-*) Seek(filvar, index);
     (*$I+*) error := IoResult;
    if error = 0 then begin
            (*$I-*) BlockRead(filvar, Buffer, nbytes, nresult);
            (*$I+*) error := IoResult;
    end;
    if error = 0 then IREADbyte := nresult (*number of bytes read*)
    else begin
      case error of
        1..99 : IREADbyte := -error; (*dos errors*)
      else IREADbyte := -2;       (* eof *)
      end;
    end;
  end;

  function IWRITbyte(nbytes : Longint; var Buffer;
                     index : LongInt; var filvar : file) : LongInt;
(*writes specified number <nbytes> of bytes from buffer <BUFFER>
         to file <filvar> located at file position
         specified by byte-index <index>
         *)
  var
    error : Integer;
    nresult : Longint;

  begin
    nresult := 0;
    filemode:=2;
     (*$I-*) Seek(filvar, index);
     (*$I+*) error := IoResult;
    if error = 0 then begin
            (*$I-*) BlockWrite(filvar, Buffer, nbytes, nresult);
            (*$I+*) error := IoResult;
    end;
    if error = 0 then IWRITbyte := nresult (*number of bytes written*)
    else begin
      case error of
        1..99 : IWRITbyte := -error; (*dos errors*)
      else IWRITbyte := -1;
      end;
    end;
  end;


  function getFilenumber(num:word):string;
  { converts a number fom 0 to 999 in a three-digit string}
  { 000,001,..999}
  var
  name : string[3];
  j: integer;
  begin
    Str(num:3, name);
    repeat
      j := Pos(' ', name);
      if j > 0 then begin
        Delete(name, j, 1);
        Insert('0', name, j);
      end;
    until j = 0;
    getfilenumber:=name;
  end;

  function getnumber(num:longint;digits:byte):string;
  { converts a number <num> in a string with <digits> digits}
  { <digits>=4: 0000,0001,..9999}
  var
  name : string[14];
  j: integer;
  begin
    Str(num:digits, name);
    repeat
      j := Pos(' ', name);
      if j > 0 then begin
        Delete(name, j, 1);
        Insert('0', name, j);
      end;
    until j = 0;
    getnumber:=name;
  end;

function FileCopy(infilename,outfilename:string) : Integer;
  {copies file with name <infilename> to specified file
   with name <outfilename>.
   Existing files are overwritten !
   Returned function code:
   0 : copy successful
   <0: error during copy }

  const
    bufsize = 32768;
  type
    buffertype = array[1..bufsize] of Byte;
  var
    error, i, j, irep, nrep, nrest : Integer;
    fpos, fdest, jb, fsize : LongInt;
    sourcefile, destfile : file;
    Buffer : ^buffertype;

    procedure FileCopy_exit;
    begin
      FreeMem(Buffer, SizeOf(Buffer^));
    end;

  begin
    error := 0;
    FileCopy:=-1;
    GetMem(Buffer, SizeOf(Buffer^));
    for i:=0 to 1 do begin
      if (buffer=nil) then begin
        showMessage('not enough memory available');
        exit;
      end;
    end;
    j := ICREATE(destfile, outfilename, ByteWrite);
    if j < 0 then begin
      FileCopy:=-1;
      FileCopy_exit; Exit;
    end;

    Assign(SourceFile,Infilename);
    j := IOPEN(SourceFile, ByteRead);
    if j < 0 then begin
      FileCopy:=-1;
      FileCopy_exit; Exit;
    end;
    fsize:= filesize(Sourcefile);
    nrep := fsize div bufsize;
    fpos := 0;
    fdest := 0;
    nrest := fsize mod bufsize;
      {WriteLn('bytes to copy: ', studybytes);}
    for irep := 1 to nrep do begin
        jb := IREADbyte(bufsize, Buffer^, fpos, Sourcefile);
        if jb = 0 then begin
          ShowMessage('read error');
        error := 1; end;
        if IWRITbyte(jb, Buffer^, fdest, destfile)
        < jb then begin
          error := 1;
        end;
        fpos := fpos+bufsize;
        fdest := fdest+bufsize;
    end;
    if nrest > 0 then begin
        jb := IREADbyte(nrest, Buffer^, fpos, Sourcefile);
        if jb = 0 then error := 1;
        if IWRITbyte(jb, Buffer^, fdest, destfile)
        < jb then error := 1;
    end;
    {$I-} Close(Sourcefile); Close(destfile); {$I+}

    if error > 0 then begin
      {Write(' Data transfer error');}
      FileCopy := -1;
    end
    else begin
      {Write(' transfer end');}
      FileCopy := 0;
    end;
    FileCopy_exit;
  end;                            (*FileCopy*)



function AddBackslash(s:string):string;
begin
  if s[length(s)]<>'\' then AddBackslash:=s+'\'
                       else AddBackslash:=s;
end;

{ STRING functions }
{------------------}

function stUpcase(s:shortstring):shortstring;
var i: integer;
begin
  for i:=1 to length(s) do s[i]:=upcase(s[i]);
  stUpcase:=s;
end;

function loCase(ch:char):char;
begin
  case ch of
  'A'..'Z': locase:=chr(ord(ch)+32);
  '': locase:='';
  '': locase:='';
  '': locase:='';
  else locase:=ch;
  end;
end;

function StLoCase(s:shortstring):shortstring;
var i: integer;
begin
  for i:=1 to length(s) do s[i]:=locase(s[i]);
  stLocase:=s;
end;

function Trim(S:shortstring):shortstring;
begin
  while ((S<>'') and (S[1]<=chr(32)) ) do delete(S,1,1);
  while ((S<>'') and (S[length(S)]<=chr(32)) ) do delete(S,length(s),1);
  Trim:=S;
end;

function justfilename(filename:string):string;
begin
  justfilename:=extractfilename(filename);
end;

function justpathname(filename:string):string;
begin
  justpathname:=extractfilepath(filename);
end;


procedure RemoveBlanks(var s : ShortString);
  { deletes blank characters in string}
  var
    j : Integer;
  begin
    repeat
      j := Pos(' ', s);
      if j > 0 then Delete(s, j, 1);
    until j = 0;
  end;

function nocomma(s:string): string;
var
  j: integer;
begin
  repeat
    j:=pos(',',s);
    if j>0 then s[j]:=' ';
  until j=0;
  nocomma:=s;
end;

function extractword(n:byte; S:string; WordDelims:charset):string;
var
  i,k,p1,p2:integer;
  ch: char;
begin
  i:=0;k:=0;p1:=0;p2:=0;
  while ((i<length(S)) and (k<n)) do
  begin
   repeat
    inc(i);
    ch:=S[i];
   until ((ch in WordDelims) or (i>=length(S)));
   p1:=p2;
   if (ch in WordDelims) then begin p2:=i; inc(k); end
   else p2:=i+1;
  end;
  extractword:= copy(S,p1+1,p2-p1-1);
end;


begin
  Progdir:=Addbackslash(Extractfiledir(Application.exename));
end.
