unit astro1;
// FITS Bildanalyse
// O. Nickel, Mainz 2005
// Astronomische Routinen

interface
uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls, ExtCtrls,Gmlib;

(*---------------------------------------------------------------------------*)
(* MJD: Modifiziertes Julianisches Datum  = JD-2400000.5                                   *)
(*      gueltig fuer jedes Datum seit 4713 v.Chr.                            *)
(*      julianischer Kalender bis zum 4. Oktober 1582                        *)
(*      gregorianischer Kalender ab dem 15. Oktober 1582                     *)
(*---------------------------------------------------------------------------*)
FUNCTION MJD(DAY,MONTH,YEAR:INTEGER;HOUR:double):double;
// JD: Julianisches Datum
FUNCTION JD(DAY,MONTH,YEAR:INTEGER;HOUR:double):double;

function convertdate(s:string;var year,month,day:integer):integer;
function converttime(s:string;var h,m:integer; var sec:single):integer;
(* CALDAT: Bestimmung des Kalenderdatums                                     *)
(*         aus dem Modifizierten Julianischen Datum (MJD)                    *)
(*---------------------------------------------------------------------------*)
PROCEDURE CALDAT(MJD:double; VAR DAY,MONTH,YEAR:INTEGER;VAR HOUR:double);



implementation

FUNCTION MJD(DAY,MONTH,YEAR:INTEGER;HOUR:double):double;
  VAR A: double; B: INTEGER;
BEGIN
    A:=10000.0*YEAR+100.0*MONTH+DAY;
    IF (MONTH<=2) THEN BEGIN MONTH:=MONTH+12; YEAR:=YEAR-1 END;
    IF (A<=15821004.1)
      THEN B:=-2+TRUNC((YEAR+4716)/4)-1179
      ELSE B:=TRUNC(YEAR/400)-TRUNC(YEAR/100)+TRUNC(YEAR/4);
    A:=365.0*YEAR-679004.0;
    MJD:=A+B+TRUNC(30.6001*(MONTH+1))+DAY+HOUR/24.0;
END;

FUNCTION JD(DAY,MONTH,YEAR:INTEGER;HOUR:double):double;
begin
  JD:=MJD(DAY,MONTH,YEAR,HOUR)+2400000.5;
end;
(* CALDAT: Bestimmung des Kalenderdatums                                     *)
(*         aus dem Modifizierten Julianischen Datum (MJD)                    *)
(*---------------------------------------------------------------------------*)
PROCEDURE CALDAT(MJD:double; VAR DAY,MONTH,YEAR:INTEGER;VAR HOUR:double);
  VAR B,D,F     : INTEGER;
      JD,JD0,C,E: double;
  BEGIN
    JD  := MJD + 2400000.5;
    (* JD0 := TRUNC(JD+0.5); *)      (* Standard Pascal  *)
    (* JD0 := INT(JD+0.5);   *)      (* TURBO Pascal     *)
    JD0 := INT(JD+0.5);
    IF (JD0<2299161.0)                            (* Kalender:        *)
      THEN BEGIN C:=JD0+1524.0 END                (* -> julianisch    *)
      ELSE BEGIN                                  (* -> gregorianisch *)
             B:=TRUNC((JD0-1867216.25)/36524.25);
             C:=JD0+(B-TRUNC(B/4))+1525.0
           END;
    D    := TRUNC((C-122.1)/365.25);          E     := 365.0*D+TRUNC(D/4);
    F    := TRUNC((C-E)/30.6001);
    DAY  := TRUNC(C-E+0.5)-TRUNC(30.6001*F);  MONTH := F-1-12*TRUNC(F/14);
    YEAR := D-4715-TRUNC((7+MONTH)/10);       HOUR  := 24.0*(JD+0.5-JD0);
  END;

function convertdate(s:string;var year,month,day:integer):integer;
var
  err: integer;
begin
  convertdate:=-1;
  if s[5]='-' then
  begin
    val(copy(s,1,4),year,err);
    if err<>0 then exit;
    val(copy(s,6,2),month,err);
    if err<>0 then exit;
    val(copy(s,9,2),day,err);
    if err<>0 then exit;
    convertdate:=0;
  end else
  if s[3]='/' then
  begin
    val(copy(s,1,2),day,err);
    if err<>0 then exit;
    val(copy(s,4,2),month,err);
    if err<>0 then exit;
    val(copy(s,7,2),year,err);
    if err<>0 then exit;
    if year>20 then year:=year+1900 else year:=year+2000; // ??
    convertdate:=0;
  end;
end;

function converttime(s:string;var h,m:integer; var sec:single):integer;
var
  err:integer;
  S1: shortstring;
begin
  Converttime:=-1;
  if s[3]<>':' then exit;
  val(copy(s,1,2),h,err);
  if err<>0 then exit;
  if s[6]<>':' then exit;
  val(copy(s,4,2),m,err);
  if err<>0 then exit;
  s1:=copy(s,7,length(s)-6);
  removeblanks(s1);
  val(s1,sec,err);
  if err<>0 then exit;
  Converttime:=0;
end;

end.


